//
// This file is part of Nolimips, a MIPS simulator with unlimited registers
// Copyright (C) 2003, 2004 Benoit Perrot <benoit@lrde.epita.fr>
//
// Nolimips is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// Nolimips is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Nolimips.  If not, see <http://www.gnu.org/licenses/>.
//
#ifndef MISC_TABLE_HH
# define MISC_TABLE_HH

# include <iostream>
# include <list>
# include <map>

# include "misc/contract.hh"

namespace misc
{

  template <class Key_T, class Entry_T>
  class Table
  {
  public:
    typedef std::map<Key_T, Entry_T>    scope_type;
    typedef std::list<scope_type>       scopes_type;

    Table ()
    {
      scopes_.push_front (scope_type ());
    }

  public:
    void                put (Key_T key, Entry_T value)
    {
      scopes_.front ()[key] = value;
    }

    bool                has (Key_T key) const
    {
      typename scope_type::const_iterator im = scopes_.front ().find (key);
      return (im != scopes_.front ().end ());
    }

    Entry_T             get (Key_T key) const
    {
      typename scope_type::const_iterator im = scopes_.front ().find (key);
      assertion (im != scopes_.front ().end ());
      return (*im).second;
    }


  public:
    void begin_scope ()
    {
      scopes_.push_front (scope_type ());
    }

    void end_scope ()
    {
      precondition(!scopes_.empty ());
      scopes_.pop_front ();
    }


  public:
    void write(std::ostream& ostr) const
    {
      bool first = true;

      for (typename scopes_type::const_iterator
             s = scopes_.begin (); s != scopes_.end (); ++s)
        {
          if (!first)
            ostr << "------------------------------" << std::endl;
          for (typename scope_type::const_iterator
                 i = s->begin (); i != s->end (); ++i)
            ostr << i->first << " = " << i->second << std::endl;
          first = false;
        }
      ostr << "==============================" << std::endl;
    }

  protected:
    scopes_type                 scopes_;
  };

  template<class K, class E>
  inline std::ostream& operator<< (std::ostream& ostr, const Table<K, E>& t)
  {
    t.write(ostr);
    return ostr;
  }

} // namespace misc

#endif // !MISC_TABLE_HH
